// ─────────────────────────────────────────────────────────────────────────────
// Variables globales
// ─────────────────────────────────────────────────────────────────────────────
let isExtensionActive = true;
let areStatsActive = false;
let originalTabId = null;
let loginTabId = null;

const AUTH_LOGIN_URL = "https://prisms.lezinter.net/fr/login";
const AUTH_BALEX_URL = "https://prisms.lezinter.net/fr/headquarters/balex";

// ─────────────────────────────────────────────────────────────────────────────
// Logs de démarrage
// ─────────────────────────────────────────────────────────────────────────────
log("🚀 ff2BaLex (background) chargé.");

browser.runtime.onInstalled.addListener((details) => {
  log("🔔 Extension installée ou mise à jour. Raison :", details.reason);
});

browser.runtime.onStartup.addListener(() => {
  log("🔄 Extension démarrée (onStartup).");
});

// ─────────────────────────────────────────────────────────────────────────────
// Suivi des changements dans le stockage
// ─────────────────────────────────────────────────────────────────────────────
browser.storage.onChanged.addListener((changes) => {
  if (changes.extensionActive) {
    isExtensionActive = changes.extensionActive.newValue;
    log("✅ Extension activée :", isExtensionActive);
  }
  if (changes.statsActive) {
    areStatsActive = changes.statsActive.newValue;
    log("📊 Statistiques activées :", areStatsActive);
  }
  refreshAllUI();
});

browser.storage.onChanged.addListener((changes, area) => {
  if (area === "local" && changes.accessToken) {
    const newToken = changes.accessToken.newValue;
    if (newToken) {
      browser.storage.local.get("extensionActive").then(({ extensionActive }) => {
        if (!extensionActive) {
          log("Token ajouté, activation automatique de l'extension.");
          browser.storage.local.set({ extensionActive: true });
          updateExtension();
          browser.runtime.sendMessage({
            action: "updateUI",
            extensionActive: true,
            isTrackingActive: true,
            autoAdd: true
          });
        }
      });
    }
  }
});

// ─────────────────────────────────────────────────────────────────────────────
// Fonctions utilitaires
// ─────────────────────────────────────────────────────────────────────────────
async function isUserConnected() {
  const { accessToken } = await browser.storage.local.get("accessToken");
  return !!accessToken;
}

async function refreshAllUI() {
  log("🔄 Rafraîchissement global de l'UI...");
  try {
    await browser.runtime.sendMessage({ action: "refreshUI" });
  } catch (error) {
    console.warn("Aucun récepteur pour 'refreshUI' :", error);
  }
}

// ─────────────────────────────────────────────────────────────────────────────
// Fonctions d'authentification & de redirection
// ─────────────────────────────────────────────────────────────────────────────
browser.runtime.onConnect.addListener((port) => {
  if (port.name === "auth") {
    port.onMessage.addListener(async (message) => {
      if (message.action === "toggleAuth") {
        log("🔄 toggleAuth reçu via port dans le background.");
        const isConnected = await isUserConnected();
        if (isConnected) {
          await disconnectFromLexicalDB();
        } else {
          actuallyOpenLoginPage();
        }
      }
    });
  }
});

async function actuallyOpenLoginPage() {
  log("🔗 Ouverture de la page de connexion.");

  // Mémoriser l'onglet actif
  const [currentTab] = await browser.tabs.query({ active: true, currentWindow: true });
  if (currentTab) {
    originalTabId = currentTab.id;
    log("✅ Onglet courant mémorisé, ID =", originalTabId);
  }
  // Ouvre un nouvel onglet pour la page de connexion et l'active
  const loginTab = await browser.tabs.create({ url: AUTH_LOGIN_URL, active: true });
  loginTabId = loginTab.id;
  log("✅ Onglet de login créé, ID =", loginTabId);
  // Notifie que l'authentification est en cours
  browser.runtime.sendMessage({ action: "authStatusChanged", isLoggedIn: false });
}

async function disconnectFromLexicalDB() {
  await browser.storage.local.remove("accessToken");
  log("🔓 Token supprimé avec succès.");

  await browser.storage.local.remove("lexiconColors");

  // Désactivation automatique de l'extension
  await browser.storage.local.set({ extensionActive: false });
  disableExtensionFeatures();
  browser.runtime.sendMessage({
    action: "updateUI",
    extensionActive: false,
    isTrackingActive: false,
    autoAdd: false
  });

  setTimeout(async () => {
    await refreshAllUI();
  }, 500);
}

async function saveToken(token) {
  log("✅ Sauvegarde du token :", token);
  await browser.storage.local.set({ accessToken: token });

  if (loginTabId) {
    try {
      await browser.tabs.remove(loginTabId);
      log("🗙 Onglet de login fermé après connexion réussie.");
    } catch (err) {
      console.warn("Impossible de fermer l'onglet de login :", err);
    }
    loginTabId = null;
  }
  if (originalTabId) {
    try {
      await browser.tabs.update(originalTabId, { active: true });
      log("🔙 Retour sur l'onglet initial :", originalTabId);
    } catch (err) {
      console.warn("Impossible de basculer sur l'onglet initial :", err);
    }
    originalTabId = null;
  }

  // Activer automatiquement l'extension
  const { extensionActive } = await browser.storage.local.get("extensionActive");
  if (!extensionActive) {
    await browser.storage.local.set({ extensionActive: true });
    updateExtension();
    browser.runtime.sendMessage({
      action: "updateUI",
      extensionActive: true,
      isTrackingActive: true,
      autoAdd: true
    });
  }
  await refreshAllUI();
}

// ─────────────────────────────────────────────────────────────────────────────
// Gestion des messages reçus 
// ─────────────────────────────────────────────────────────────────────────────
browser.runtime.onMessage.addListener(async (message, sender, sendResponse) => {
  log("📩 Message reçu dans background.js :", message);

  switch (message.action) {
    case "toggleAuth": {
      const isConnected = await isUserConnected();
      if (isConnected) {
        await disconnectFromLexicalDB();
        await browser.storage.local.remove("lexiconColors");
      } else {
        actuallyOpenLoginPage();
      }
      break;
    }
    case "getDefinitionWiki": {
      if (message.selectedText && message.selectedText.trim() !== "") {
        log("🌐 Requête Wiktionnaire pour :", message.selectedText);
        const definition = await window.fetchWiktionaryDefinition(message.selectedText.trim());
        browser.runtime.sendMessage({
          action: "fetchWiktionaryDefinitionResponse",
          selectedText: message.selectedText,
          definitions: [{
            source: "Wiktionnaire",
            text: definition,
          }],
        });
      } else {
        console.warn("⚠️ Texte sélectionné vide. Annulation de la requête.");
      }
      break;
    }
    case "checkAuthStatus": {
      const connected = await isUserConnected();
      sendResponse(connected);
      break;
    }
    case "authStatusChanged": {
      log("🔄 Mise à jour de l'état d'authentification :", message.isLoggedIn);
      break;
    }
    case "saveToken": {
      if (message.token) {
        await saveToken(message.token);
      } else {
        console.warn("⚠️ Aucune valeur de token reçue.");
      }
      break;
    }
    case "toggleLexiconHighlight": {
      const tabs = await browser.tabs.query({active: true, currentWindow: true});
      if (tabs[0]) {
        try {
          // S'assurer que le script est injecté
          await browser.scripting.executeScript({
              target: { tabId: tabs[0].id },
              files: ["src/utils/highlighting.js"]
          });

          // Envoyer le message d'activation
          await browser.tabs.sendMessage(tabs[0].id, {
            command: message.isActive ? "activate-highlighting" : "deactivate-highlighting",
            lexiconId: message.lexiconId
          });
          log(`✅ Message de surlignage transmis à l'onglet ${tabs[0].id}`);
        } catch (error) {
          log("❌ Erreur lors de la gestion du surlignage:", error);
        }
      }
      break;
    }

    case "register-highlighting-script": {
      log("📝 Script de surlignage enregistré pour l'onglet", sender.tab.id);
      break;
    }
    
    default:
      break;
  }
  return true;
});

// ─────────────────────────────────────────────────────────────────────────────
// Web Navigation : Injection de scripts et récupération du token
// ─────────────────────────────────────────────────────────────────────────────
browser.webNavigation.onCompleted.addListener(async (details) => {
  const url = new URL(details.url);

  // Injection d'un popup d'instruction sur la page de login
  if (url.hostname === "prisms.lezinter.net" && url.pathname === "/fr/login") {
    log("📘 Injection du popup d'instruction sur la page de login Prisms.");
    showInstructionPopup(details);
  }
  // Récupération du token sur la page /balex
  if (url.hostname === "prisms.lezinter.net" && url.pathname === "/fr/headquarters/balex") {
    log("🟢 Page /balex détectée. Tentative de récupération du token.");
    try {
      await new Promise(resolve => setTimeout(resolve, 3000));
      await browser.tabs.executeScript(details.tabId, {
        code: `
          (function() {
            log("🔍 Recherche du token...");
            const tokenElement = document.getElementById("accessToken") || document.getElementById("accesToken");
            if (tokenElement) {
              const token = tokenElement.innerText.trim();
              log("🔐 Token détecté :", token);
              browser.runtime.sendMessage({ action: "saveToken", token });
            } else {
              log("❌ Token introuvable.");
            }
            return null;
          })();
        `
      });
    } catch (error) {
      log("❌ Erreur lors de la récupération du token :", error);
    }
  }
}, { url: [{ hostContains: "prisms.lezinter.net" }] });

// ─────────────────────────────────────────────────────────────────────────────
// Web Request : Redirection automatique vers /balex
// ─────────────────────────────────────────────────────────────────────────────
browser.webRequest.onBeforeRequest.addListener(
  function(details) {
    if (details.url === "https://prisms.lezinter.net/fr/headquarters/") {
      log("🚀 Redirection automatique vers /balex.");
      return { redirectUrl: AUTH_BALEX_URL };
    }
  },
  { urls: ["https://prisms.lezinter.net/fr/headquarters/*"] },
  ["blocking"]
);

// ─────────────────────────────────────────────────────────────────────────────
// Affichage d'un popup d'instruction sur /fr/login
// ─────────────────────────────────────────────────────────────────────────────
function showInstructionPopup(details) {
  browser.tabs.executeScript(details.tabId, {
    code: `
      if (!document.getElementById("balex-instruction-popup")) {
        const popup = document.createElement("div");
        popup.id = "balex-instruction-popup";
        popup.style.position = "fixed";
        popup.style.top = "50%";
        popup.style.left = "50%";
        popup.style.transform = "translate(-50%, -50%)";
        popup.style.backgroundColor = "#a08e9f";
        popup.style.color = "#323046";
        popup.style.padding = "12px";
        popup.style.borderRadius = "10px";
        popup.style.boxShadow = "0 2px 10px rgba(0, 0, 0, 0.3)";
        popup.style.zIndex = "10000";
        popup.style.fontFamily = "Luciole";
        popup.style.fontSize = "16px";
        popup.style.width = "300px";
        popup.style.textAlign = "center";
  
        popup.innerHTML = \`
          <h5 style="color: #fff; font-weight: bold; margin-top: 2px;">Connexion à l'extension</h5>
          <p style="margin: 8px 0;">
            Après avoir renseigné vos identifiants, veuillez cliquer sur 
            <strong>"Se connecter avec BaLex"</strong>.
          </p>
          <button id="close-popup-btn" style="
            width: 100%;
            margin-top: 15px;
            padding: 8px;
            border: none;
            background-color: #8d5c70;
            color: #fbfcfc;
            font-weight: bold;
            cursor: pointer;
            border-radius: 5px;
          ">Fermer</button>
        \`;
  
        document.body.appendChild(popup);
  
        const closeBtn = document.getElementById("close-popup-btn");
        closeBtn.onclick = () => popup.remove();
      }
    `
  });
}

// ─────────────────────────────────────────────────────────────────────────────
// Initialisation du WebWorker
// ─────────────────────────────────────────────────────────────────────────────
let worker = null;

function initWorker() {
  if (!worker) {
    log("[Background] Initialisation du WebWorker...");
    try {
      worker = new Worker(browser.runtime.getURL("src/workers/pyodide_worker.js"));
      // Centralisation de l'écoute des messages et erreurs du Worker
      worker.addEventListener("message", handleWorkerMessage);
      worker.addEventListener("error", handleWorkerError);
      log("[Background] WebWorker initialisé avec succès.");
    } catch (error) {
      log("[Background] Échec de l'initialisation du WebWorker :", error);
    }
  }
}

function handleWorkerError(error) {
  log("Erreur du WebWorker :", error.message);
}

function handleWorkerMessage(event) {
  const data = event.data;
  log("[Background] Message du WebWorker :", data);

  switch (data.type) {
    case "pyodide-simplemma":
      if (data.status === "success") {
        log("[Background] Pyodide et Simplemma prêts. Mise à jour de l'état.");
        browser.storage.local.set({ pyodideSimplemmaReady: true });
        checkAndUpdateTracking();
      } else if (data.status === "error") {
        log("[Background] Erreur lors du chargement :", data.message);
      } else if (data.status === "already_loaded") {
        log("[Background] Pyodide et Simplemma déjà chargés.");
      }
      break;
    case "update-frequencies":
      log("[Background] Mise à jour des fréquences :", data.frequencies);
      notifyAllTabs({ command: "update-frequencies", frequencies: data.frequencies });
      // Si un port stats est connecté, vous pouvez lui transmettre également :
      // port.postMessage({ command: "update-frequencies", frequencies: data.frequencies });
      browser.storage.local.set({ lemmaFrequencies: data.frequencies });
      break;
    case "threshold-exceeded":
      log("[Background] Mots dépassant le seuil :", data.wordsAboveThreshold);
      notifyAllTabs({ command: "threshold-exceeded", wordsAboveThreshold: data.wordsAboveThreshold });
      break;
    default:
      console.warn("[Background] Message non traité du Worker :", data);
      break;
  }
}

// Initialisation du worker dès le démarrage
initWorker();

// ─────────────────────────────────────────────────────────────────────────────
// Écoute des messages de la popup et transmission au WebWorker
// ─────────────────────────────────────────────────────────────────────────────
browser.runtime.onMessage.addListener(async (message, sender, sendResponse) => {
  log("[Background] Message reçu :", message);

  if (!worker) {
    initWorker();
  }

  if (message.command === "toggle-stats") {
    log(`[Background] Statistiques ${message.isActive ? "activées" : "désactivées"}`);
    const { isActive } = message;
    await browser.storage.local.set({ isTrackingActive: isActive });
    if (!isActive) {
      await browser.storage.local.set({ autoAdd: false });
    }
    checkAndUpdateTracking();
  }

  if (message.command === "pyodide-simplemma") {
    log("[Background] Demande d'initialisation de Pyodide et Simplemma...");
    worker.postMessage({ command: "pyodide-simplemma" });
  }

  return true;
});


// ─────────────────────────────────────────────────────────────────────────────
// Chargement et sauvegarde des fréquences stockées
// ─────────────────────────────────────────────────────────────────────────────
async function loadStoredFrequencies() {
  const { storedFrequencies } = await browser.storage.local.get("storedFrequencies");
  return storedFrequencies || {};
}

let storedFrequencies = {};

loadStoredFrequencies().then(frequencies => {
  storedFrequencies = frequencies;
  log("[Background] Fréquences initialisées :", storedFrequencies);
});

// ─────────────────────────────────────────────────────────────────────────────
// Statistiques : Vérification et activation/désactivation du tracking
// ─────────────────────────────────────────────────────────────────────────────
async function checkAndUpdateTracking() {
  const { isTrackingActive, pyodideSimplemmaReady } = await browser.storage.local.get(["isTrackingActive", "pyodideSimplemmaReady"]);
  if (isTrackingActive && pyodideSimplemmaReady) {
    log("[Background] Activation du tracking.");
    notifyAllTabs({ command: "activate-stats" });
  } else {
    log("[Background] Désactivation du tracking.");
    notifyAllTabs({ command: "deactivate-stats" });
  }
}

// ─────────────────────────────────────────────────────────────────────────────
// Statistiques : Gestion des onglets
// ─────────────────────────────────────────────────────────────────────────────
async function notifyAllTabs(message) {
  browser.tabs.query({}).then((tabs) => {
    tabs.forEach((tab) => {
      browser.tabs.sendMessage(tab.id, message)
        .catch((error) => console.warn(`[Background] Impossible d'envoyer un message à l'onglet ${tab.id} : ${error}`));
    });
  });
}

// ─────────────────────────────────────────────────────────────────────────────
// Statistiques : Écoute des modifications du stockage et mise à jour du tracking
// ─────────────────────────────────────────────────────────────────────────────
browser.storage.onChanged.addListener(async (changes, area) => {
  if (area === "local" && (changes.isTrackingActive || changes.pyodideSimplemmaReady)) {
    checkAndUpdateTracking();
  }
  if (area === "local" && (changes.accessToken || changes.threshold || changes.trackedLanguages || changes.autoAdd)) {
    log("[Background] Mise à jour des préférences détectée.");
    const { accessToken, trackedLanguages, threshold, autoAdd } = await browser.storage.local.get([
      "accessToken", "trackedLanguages", "threshold", "autoAdd"
    ]);
    const isAuthenticated = !!accessToken;
    worker.postMessage({
      command: "update-preferences",
      isAuthenticated,
      trackedLanguages: trackedLanguages || [],
      threshold: threshold || 10,
      autoAdd: autoAdd || false
    });
  }
});

// ─────────────────────────────────────────────────────────────────────────────
// Écoute de l'inscription de stats.js
// ─────────────────────────────────────────────────────────────────────────────
browser.runtime.onMessage.addListener(async (message, sender) => {
  if (message.command === "register-stats-script") {
    log("[Background] stats.js s'est enregistré.");
    const { isTrackingActive, pyodideSimplemmaReady } = await browser.storage.local.get(["isTrackingActive", "pyodideSimplemmaReady"]);
    if (isTrackingActive && pyodideSimplemmaReady) {
      browser.tabs.sendMessage(sender.tab.id, { command: "activate-stats" })
        .catch((error) => console.warn(`[Background] Impossible d'envoyer activate-stats à ${sender.tab.id} : ${error}`));
    }
  }
});

// ─────────────────────────────────────────────────────────────────────────────
// Connexion entre stats.js et le Worker via un port dédié
// ─────────────────────────────────────────────────────────────────────────────
browser.runtime.onConnect.addListener((port) => {
  log("[Background] Connexion établie :", port.name);
  if (port.name === "stats-worker-port") {
    // Redirige les messages de stats.js vers le Worker
    port.onMessage.addListener((message) => {
      log("[Background] Message reçu de stats.js :", message);
      worker.postMessage(message);
    });

    // Transmettre les réponses du Worker à stats.js via le port
    worker.addEventListener("message", (event) => {
      const data = event.data;
      if (data.type === "update-frequencies") {
        port.postMessage({ command: "update-frequencies", frequencies: data.frequencies });
      }
      if (data.type === "threshold-exceeded") {
        port.postMessage({ command: "threshold-exceeded", wordsAboveThreshold: data.wordsAboveThreshold });
      }
    });
  }
});

// ─────────────────────────────────────────────────────────────────────────────
// Stoplists : Chargement et envoi au Worker
// ─────────────────────────────────────────────────────────────────────────────
let stoplistFr = [];

function loadStoplist() {
  fetch(browser.runtime.getURL("stoplist_fr.txt"))
    .then(response => response.text())
    .then(text => {
      stoplistFr = text.split("\n").map(word => word.trim());
      log("[Background] Stoplist chargée :", stoplistFr);
      sendStoplistToWorker();
    })
    .catch(error => log("[Background] Erreur lors du chargement de la stoplist :", error));
}

function sendStoplistToWorker() {
  log("[Background] Envoi de la stoplist au Worker...");
  worker.postMessage({ command: "update-stoplist", stoplist: stoplistFr });
}

browser.runtime.onStartup.addListener(loadStoplist);
browser.runtime.onInstalled.addListener(loadStoplist);

async function handleHighlighting(command, lexiconId, tabId) {
  log(`🎯 Gestion du surlignage: ${command} pour le lexique ${lexiconId}`);
  
  try {
      // S'assurer que le script est injecté
      await browser.scripting.executeScript({
          target: { tabId: tabId },
          files: ["utils/highlighting.js"]
      });

      // Envoyer le message d'activation
      const response = await browser.tabs.sendMessage(tabId, {
          command: command,
          lexiconId: lexiconId
      });
      
      log("✅ Réponse du content script:", response);
      return response;
  } catch (error) {
      log("❌ Erreur lors de la gestion du surlignage:", error);
      return false;
  }
}