log("✅ popup.js chargé avec succès !");

// ==========================
// Fonctions utilitaires
// ==========================
async function getAccessToken() {
  const { accessToken } = await browser.storage.local.get("accessToken");
  return accessToken;
}

// ==========================
// Gestion de la connexion
// ==========================
async function updateConnectionButton() {
  const accessToken = await getAccessToken();
  const button = document.getElementById("auth-button");

  if (!button) {
    log("❌ Le bouton de connexion n'a pas été trouvé.");
    return;
  }

  if (accessToken) {
    button.textContent = "Se déconnecter";
    button.style.position = "relative";
    button.className = "tooltip-container";
    const tooltip = document.createElement("span");
    tooltip.className = "tooltip";
    tooltip.textContent = "En vous déconnectant, vous perdrez l'accès à vos lexiques personnels, ainsi qu'aux fonctionnalités d'ajout automatique et de statistiques d'utilisation.";
    button.appendChild(tooltip);
  } else {
    button.textContent = "Se connecter";
    button.style.position = "relative";
    button.className = "tooltip-container";
    const tooltip = document.createElement("span");
    tooltip.className = "tooltip";
    tooltip.textContent = "En vous connectant, vous pourrez accéder à vos lexiques personnels, ainsi qu'aux fonctionnalités d'ajout automatique et de statistiques d'utilisation.";
    button.appendChild(tooltip);
  }

  button.onclick = async () => {
    await browser.runtime.sendMessage({ action: "toggleAuth" });
  };
}

// ==========================
// Gestion de la sélection des langues
// ==========================
async function updateLanguageSelection() {
  const languageSelection = document.getElementById("language-selection");
  languageSelection.innerHTML = "<p id='loading-languages' style='color: gray;'>Chargement...</p>";

  const accessToken = await getAccessToken();
  if (!accessToken) {
    languageSelection.innerHTML = "<p style='color: red;'>Veuillez vous connecter.</p>";
    return;
  }

  const lexicons = await getLexicons(accessToken);
  const userLanguages = [...new Set(lexicons.map(lex => lex.language))];

  const { trackedLanguages } = await browser.storage.local.get("trackedLanguages") || { trackedLanguages: [] };

  languageSelection.innerHTML = "";
  if (userLanguages.length === 0) {
    languageSelection.innerHTML = "<p style='color: red;'>Aucun lexique personnel trouvé.</p>";
    return;
  }

  userLanguages.forEach(lang => {
    const langButton = document.createElement("div");
    langButton.classList.add("lang-option");
    langButton.textContent = lang.toUpperCase();
    langButton.dataset.value = lang;
    if (trackedLanguages && trackedLanguages.includes(lang)) {
      langButton.classList.add("selected");
    }
    langButton.addEventListener("click", () => {
      langButton.classList.toggle("selected");
    });
    languageSelection.appendChild(langButton);
  });

  log("✅ Sélection des langues mise à jour avec :", userLanguages);
}

// ==========================
// Gestion principale de l'extension
// ==========================
async function updateExtension() {
  // Récupération de tous les états nécessaires
  const accessToken = await getAccessToken();
  const { 
    extensionActive, 
    isTrackingActive, 
    autoAdd, 
    threshold,
    pyodideSimplemmaReady 
  } = await browser.storage.local.get([
    "extensionActive",
    "isTrackingActive",
    "autoAdd",
    "threshold",
    "pyodideSimplemmaReady"
  ]) || { 
    extensionActive: false, 
    isTrackingActive: false,
    autoAdd: false,
    threshold: 10,
    pyodideSimplemmaReady: false
  };

  const isLoggedIn = !!accessToken;

  // Mise à jour du bouton d'activation de l'extension
  const toggleExtensionBtn = document.getElementById("toggleExtensionBtn");
  if (toggleExtensionBtn) {
    toggleExtensionBtn.textContent = extensionActive ? "Désactiver l'analyse" : "Activer l'analyse";
    toggleExtensionBtn.style.pointerEvents = isLoggedIn ? "auto" : "none";
    toggleExtensionBtn.disabled = !isLoggedIn;
    toggleExtensionBtn.style.position = "relative";
    toggleExtensionBtn.className = "tooltip-container";

    const existingTooltipExt = toggleExtensionBtn.querySelector('.tooltip');
    if (existingTooltipExt) {
      existingTooltipExt.remove();
    }
    const tooltipExt = document.createElement("span");
    tooltipExt.className = "tooltip";
    tooltipExt.style.opacity = "1 !important";
    if (!isLoggedIn) {
      tooltipExt.textContent = "Connectez-vous pour activer l'analyse";
      tooltipExt.style.display = "block";
    } else if (!extensionActive) {
      tooltipExt.textContent = "Activer les fonctionnalités de l'extension : affichage des mots et des définitions de vos lexiques, ajout de mots, etc.";
      tooltipExt.style.display = "block";
    } else {
      tooltipExt.style.display = "none";
    }
    toggleExtensionBtn.appendChild(tooltipExt);
    
    toggleExtensionBtn.onclick = handleToggleExtension;
  }

  // Mise à jour des options de statistiques
  const statsOptions = document.getElementById("stats-options");
  const toggleStatsBtn = document.getElementById("toggleStatsBtn");
  const openStats = document.getElementById("open-stats");

  if (statsOptions) {
    statsOptions.style.display = (isLoggedIn && extensionActive) ? "block" : "none";
  }

  // Mise à jour du bouton des statistiques
  if (toggleStatsBtn) {
    const isEnabled = isLoggedIn && extensionActive;
    toggleStatsBtn.textContent = isTrackingActive ? "Désactiver les statistiques" : "Activer les statistiques";
    toggleStatsBtn.style.pointerEvents = isEnabled ? "auto" : "none";
    toggleStatsBtn.disabled = !isEnabled;
    toggleStatsBtn.style.position = "relative";
    toggleStatsBtn.className = "tooltip-container";
    
    const existingTooltipStats = toggleStatsBtn.querySelector('.tooltip');
    if (existingTooltipStats) { existingTooltipStats.remove(); }
    
    const tooltipStats = document.createElement("span");
    tooltipStats.className = "tooltip";
    tooltipStats.style.opacity = "1 !important";
    if (!isLoggedIn) {
      tooltipStats.textContent = "Connectez-vous pour accéder aux statistiques";
      tooltipStats.style.display = "block";
    } else if (!extensionActive) {
      tooltipStats.textContent = "Veuillez activer l'analyse pour utiliser les statistiques";
      tooltipStats.style.display = "block";
    } else {
      tooltipStats.style.display = "none";
    }
    toggleStatsBtn.appendChild(tooltipStats);
  }


  if (openStats) {
    openStats.style.display = (isLoggedIn && extensionActive && isTrackingActive) ? "block" : "none";
  }

  // Mise à jour des options d'ajout automatique
  const autoAddContainer = document.getElementById("auto-add")?.parentElement;
  const autoAddCheckbox = document.getElementById("auto-add");
  const autoAddOptions = document.getElementById("auto-add-options");
  const saveOptionsBtn = document.getElementById("save-options");

  if (autoAddContainer) {
    autoAddContainer.style.display = (isLoggedIn && extensionActive) ? "block" : "none";
  }

  if (autoAddCheckbox && isLoggedIn) {
    autoAddCheckbox.checked = autoAdd;
  }

  if (autoAddOptions) {
    autoAddOptions.classList.toggle("hidden", !autoAdd);
  }

  if (saveOptionsBtn) {
    saveOptionsBtn.classList.toggle("hidden", !autoAdd);
  }

  // Mise à jour du message de chargement Pyodide
  const statusContainer = document.getElementById('pyodide-loading');
  if (statusContainer) {
    if (!isLoggedIn) {
      statusContainer.innerHTML = "";
    } else if (!pyodideSimplemmaReady && extensionActive && isTrackingActive) {
      statusContainer.innerHTML = "<p style='color: black; text-align: center; font-size: 11px;'>Chargement de l'extension en cours, veuillez patienter...</p>";
    } else if (pyodideSimplemmaReady && extensionActive && isTrackingActive) {
      statusContainer.innerHTML = "<p style='color: black; text-align: center; font-size: 11px;'>C'est prêt !</p>";
      setTimeout(() => {
        statusContainer.innerHTML = "";
      }, 2000);
    } else {
      statusContainer.innerHTML = "";
    }
  }

  // Mise à jour de la sélection des langues
  await updateLanguageSelection();

  log("Interface mise à jour complètement", {
    isLoggedIn,
    extensionActive,
    isTrackingActive,
    autoAdd,
    pyodideSimplemmaReady
  });
}

async function handleToggleExtension() {
  const accessToken = await getAccessToken();
  if (!accessToken) return;
  await proceedToggleExtension();
}

async function proceedToggleExtension() {
  const { extensionActive, isTrackingActive } = await browser.storage.local.get({
    extensionActive: false,
    isTrackingActive: false
  });
  const newState = !extensionActive;
  await browser.storage.local.set({ extensionActive: newState });

  if (!newState) {
    await browser.storage.local.set({ isTrackingActive: false });
    if (isTrackingActive) window.open("stats.html", "_blank");
    browser.runtime.sendMessage({ action: "closeSidebarBlocks" });
  }

  browser.runtime.sendMessage({ action: "toggleExtension", isActive: newState });
  await updateExtension();
}

// ==========================
// Écouteurs d'événements
// ==========================
document.addEventListener("DOMContentLoaded", async () => {
  await updateConnectionButton();
  await updateExtension();
});

// Gestion des statistiques
document.getElementById("toggleStatsBtn")?.addEventListener("click", async () => {
  const accessToken = await getAccessToken();
  if (!accessToken) return;

  const current = await browser.storage.local.get("isTrackingActive");
  const newState = !current.isTrackingActive;
  await browser.storage.local.set({ isTrackingActive: newState });

  if (!newState) {
    await browser.storage.local.set({ autoAdd: false });
  }

  browser.runtime.sendMessage({ command: "toggle-stats", isActive: newState });
  if (newState) {
    browser.runtime.sendMessage({ command: "pyodide-simplemma" });
  }
  
  await updateExtension();
});

// Gestion de l'ajout automatique
document.getElementById("auto-add")?.addEventListener("change", async () => {
  const isChecked = document.getElementById("auto-add").checked;
  document.getElementById("auto-add-options").classList.toggle("hidden", !isChecked);
  document.getElementById("save-options").classList.toggle("hidden", !isChecked);
  if (!isChecked) {
    await browser.storage.local.set({ autoAdd: false });
    await updateExtension();
  }
});

// Sauvegarde des options
document.getElementById("save-options")?.addEventListener("click", async () => {
  const autoAdd = document.getElementById("auto-add").checked;
  const threshold = parseInt(document.getElementById("threshold").value, 10);
  const selectedLanguages = Array.from(document.querySelectorAll("#language-selection .lang-option.selected"))
    .map(option => option.dataset.value);

  const errorMessage = document.getElementById("error-message");
  if (autoAdd && selectedLanguages.length === 0) {
    errorMessage?.classList.remove("hidden");
    return;
  }
  errorMessage?.classList.add("hidden");

  await browser.storage.local.set({
    autoAdd,
    threshold,
    trackedLanguages: selectedLanguages
  });
  
  await updateExtension();
  log("Options sauvegardées et interface mise à jour");
});

// Ouverture des statistiques
document.getElementById("open-stats")?.addEventListener("click", () => {
  window.open("stats.html", "_blank");
});

// ==========================
// Gestion des messages et du stockage
// ==========================
browser.runtime.onMessage.addListener(async (message) => {
  log("📩 Message reçu dans popup.js :", message);
  if (message.action === "updateUI") {
    await updateExtension();
  } else if (message.action === "notify") {
    alert(message.message);
  }
});

browser.storage.onChanged.addListener((changes, area) => {
  if (area === "local") {
    updateExtension();
  }
});

// ==========================
// Gestion des notifications
// ==========================
function showNotification(message) {
  const notificationBox = document.getElementById("extension-notification");
  const notificationText = document.getElementById("notification-text");
  const closeButton = document.getElementById("close-notification");

  if (notificationBox && notificationText && closeButton) {
    notificationText.textContent = message;
    notificationBox.classList.remove("hidden");
    closeButton.addEventListener("click", () => {
      notificationBox.classList.add("hidden");
    }, { once: true });
  } else {
    log("❌ Impossible d'afficher la notification : élément manquant.");
  }
}

function handleWorkerMessage(event) {
  const data = event.data;
  log("[Background] Message du WebWorker :", data);
  
  if (data.type === "process-text" && data.status === "error") {
    browser.runtime.sendMessage({
      action: "notify",
      message: data.message
    });
    return;
  }
}
