log("✅ sidebar.js chargé !");
log(
  "🌐 Vérification API browser :",
  typeof browser !== "undefined" ? "✅ Disponible" : "❌ Non disponible"
);


// ─────────────────────────────────────────────────────────────────────────────
// ▌ Variables globales
// ─────────────────────────────────────────────────────────────────────────────
let authToken = window.authToken;
window.authToken = authToken;

// ─────────────────────────────────────────────────────────────────────────────
// ▌ Fonctions liées au token
// ─────────────────────────────────────────────────────────────────────────────
async function getAuthTokenFromStorage() {
  try {
    const { accessToken } = await browser.storage.local.get("accessToken");
    if (accessToken) {
      log(`🔑 Token récupéré depuis le stockage local : [${accessToken}]`);
      authToken = accessToken;
      window.authToken = accessToken; 
      return authToken;
    }
    log("⚠️ Aucun token trouvé dans le stockage local.");
  } catch (error) {
    log("❌ Erreur lors de la récupération du token :", error);
  }
  return null;
}

// ─────────────────────────────────────────────────────────────────────────────
// ▌ Mise à jour de l'interface utilisateur (UI)
// ─────────────────────────────────────────────────────────────────────────────
function updateAuthButton(isLoggedIn) {
  const authButton = document.getElementById("auth-button");
  if (authButton) {
    authButton.textContent = isLoggedIn ? "Se déconnecter" : "Se connecter";
    log("🔄 Bouton d'authentification mis à jour :", authButton.textContent);
  } else {
    console.warn("⚠️ Bouton d'authentification (#auth-button) introuvable.");
  }
}

function toggleElementsVisibility(isLoggedIn) {
  const elementsToShowOrHide = [
    { id: "add-to-lexiques", shouldShow: isLoggedIn },
    { id: "possible-definitions", shouldShow: isLoggedIn },
    { id: "mesLexiquesContainer", shouldShow: isLoggedIn },
  ];

  elementsToShowOrHide.forEach(({ id, shouldShow }) => {
    const element = document.getElementById(id);
    if (element) {
      element.style.display = shouldShow ? "block" : "none";
      log(`🔄 Élément #${id} ${shouldShow ? "affiché" : "masqué"}.`);
    } else {
      console.warn(`⚠️ Élément #${id} introuvable.`);
    }
  });
}

function toggleHighlightMessage(isLoggedIn) {
  const highlightNote = document.getElementById("highlight-note");
  if (highlightNote) {
    highlightNote.style.display = isLoggedIn ? "block" : "none";
  }
}

/**
 * Met à jour l'état global de la barre latérale (bouton d'authentification, etc.)
 */
async function refreshSidebarState() {
  log("🔄 Début de l'actualisation de la barre latérale...");
  const token = await getAuthTokenFromStorage();
  const isLoggedIn = !!token;

  updateAuthButton(isLoggedIn);
  toggleElementsVisibility(isLoggedIn);
  toggleHighlightMessage(isLoggedIn);

  if (isLoggedIn) {
    if (!window.lexiconColorsUpdated) {
      await updateLexiconColors(authToken);
      window.lexiconColorsUpdated = true;
    }
    await fetchLexicons();
  } else {
    // Si l'utilisateur n'est pas connecté, on ferme tous les blocs
    document.querySelectorAll('.block-content').forEach(block => {
      block.classList.add('hidden');
      // Mise à jour des boutons de bascule associés pour afficher le symbole "+"
      const header = block.previousElementSibling;
      if (header) {
        const toggleBtn = header.querySelector(".toggle-btn");
        if (toggleBtn) {
          toggleBtn.textContent = '+';
        }
      }
    });
    
    const lexiquesContainer = document.getElementById("lexiques");
    if (lexiquesContainer) {
      lexiquesContainer.textContent = "Veuillez vous connecter pour voir vos lexiques.";
      lexiquesContainer.style.textAlign = "center";
      lexiquesContainer.style.fontStyle = "italic";
    }
    const lexiconResultElement = document.getElementById("lexiconResult");
    if (lexiconResultElement) {
      lexiconResultElement.innerHTML = "";
    }
  }

  log("✅ Barre latérale actualisée. Utilisateur connecté :", isLoggedIn);
}

/**
 * Bascule l'affichage d'un bloc.
 * @param {string} blockId - L'ID du conteneur à basculer.
 * @param {HTMLElement} btn - Le bouton qui déclenche la bascule, pour mettre à jour son texte.
 */
function toggleBlock(blockId, btn) {
  const block = document.getElementById(blockId);
  if (block) {
    if (block.classList.contains("hidden")) {
      block.classList.remove("hidden");
      if (btn) btn.textContent = "–";
    } else {
      block.classList.add("hidden");
      if (btn) btn.textContent = "+";
    }
  }
}

/**
 * Ouvre un bloc s'il est fermé et met à jour le bouton de bascule.
 * @param {string} blockId - L'ID du conteneur à ouvrir.
 * @param {HTMLElement} [btn] - (Optionnel) Le bouton de bascule à mettre à jour.
 */
function openBlock(blockId, btn) {
  const block = document.getElementById(blockId);
  if (block && block.classList.contains("hidden")) {
    block.classList.remove("hidden");
    
    if (btn) {
      btn.textContent = "–";
    } else {
      const header = block.previousElementSibling;
      if (header) {
        const toggleBtn = header.querySelector(".toggle-btn");
        if (toggleBtn) {
          toggleBtn.textContent = "–";
        }
      }
    }
  }
}

function closeBlock(blockId, btn) {
  const block = document.getElementById(blockId);
  if (block && !block.classList.contains("hidden")) {
    block.classList.add("hidden");
    
    if (btn) {
      btn.textContent = "+";
    } else {
      const header = block.previousElementSibling;
      if (header) {
        const toggleBtn = header.querySelector(".toggle-btn");
        if (toggleBtn) {
          toggleBtn.textContent = "+";
        }
      }
    }
  }
}


// ─────────────────────────────────────────────────────────────────────────────
// ▌ Gestion des lexiques (Affichage)
// ─────────────────────────────────────────────────────────────────────────────

async function fetchLexicons() {
  try {
    log("🔄 Début de la récupération des lexiques...");
    log("🔑 Token utilisé :", authToken);

    if (!authToken) {
      throw new Error("⚠️ Aucun token disponible. Veuillez vous connecter.");
    }

    const lexicons = await getLexicons(authToken);
    log("📚 Réponse brute de l'API :", lexicons);

    if (!Array.isArray(lexicons) || lexicons.length === 0) {
      throw new Error("⚠️ Aucun lexique trouvé.");
    }

    lexiconMap.clear();
    lexicons.forEach((lex) => {
      let lexiconName = "";
      if (lex.category === "User") {
        lexiconName = DEBUG 
          ? `Lexique personnel : ${lex.user?.pseudo || "Inconnu"} (${lex.id})`
          : `Lexique personnel : ${lex.user?.pseudo || "Inconnu"}`;
        if (lex.language) {
          lexiconName += ` [${lex.language}]`;
        }
      } else if (lex.category === "Group") {
        lexiconName = DEBUG 
          ? `Lexique de groupe : ${lex.group?.name || "Inconnu"} (${lex.id})`
          : `Lexique de groupe : ${lex.group?.name || "Inconnu"}`
      } else {
        lexiconName = DEBUG ? `Lexique : ${lex.id}` : "Lexique" ;
      }
      lexiconMap.set(lex.id, lexiconName);
    });

    await displayLexiconsWithCheckbox(lexicons.map((lex) => ({
      lexiconName:
        lex.category === "User"
          ? DEBUG
            ? `Lexique personnel : ${lex.user?.pseudo || "Inconnu"} (${lex.id})${(lex.language) ? ` [${lex.language}]` : ""}`
            : `Lexique personnel : ${lex.user?.pseudo || "Inconnu"}${(lex.language) ? ` [${lex.language}]` : ""}`
          : DEBUG
            ? `Lexique de groupe : ${lex.group?.name || "Inconnu"} (${lex.id})${(lex.language) ? ` [${lex.language}]` : ""}`
            : `Lexique de groupe : ${lex.group?.name || "Inconnu"}${(lex.language) ? ` [${lex.language}]` : ""}`,
      lexiconId: lex.id,
      active: lex.active || false,
    })));
    
    // Restaurer l'état des boutons après l'affichage
    await restoreHighlightingState();
  } catch (error) {
    log("❌ Erreur lors du chargement des lexiques :", error.message);
    const lexiquesContainer = document.getElementById("lexiques");
    if (lexiquesContainer) {
      lexiquesContainer.textContent = error.message || "Erreur lors du chargement des lexiques.";
    }
  }
}

/**
 * Affiche la liste des lexiques avec des checkboxes dans la barre latérale.
 * @param {Array} lexicons - Liste des lexiques à afficher.
 */
async function displayLexiconsWithCheckbox(lexicons) {
  const lexiquesContainer = document.getElementById("lexiques");
  if (!lexiquesContainer) {
    console.warn("⚠️ Élément #lexiques introuvable.");
    return;
  }
  lexiquesContainer.innerHTML = "";
  if (lexicons.length === 0) {
    log("⚠️ Aucun lexique à afficher.");
    lexiquesContainer.textContent = "Aucun lexique disponible.";
    return;
  }
  
  for (const { lexiconName, lexiconId, active } of lexicons) {
    const lexiqueDiv = document.createElement("div");
    lexiqueDiv.className = "lexique-item";
    
    const color = await getColorForLexicon(lexiconId);
    const circleIcon = createColorCircle(color, 24);
    
    const iconDiv = document.createElement("div");
    iconDiv.className = "lexique-icon";
    iconDiv.appendChild(circleIcon);
    
    const labelSpan = document.createElement("span");
    labelSpan.className = "lexique-label";
    labelSpan.textContent = lexiconName;
    
    // Conteneur pour la checkbox (avec tooltip)
    const checkboxContainer = document.createElement("label");
    checkboxContainer.className = "tooltip-container lexique-checkbox-container";
    const addCheckbox = document.createElement("input");
    addCheckbox.type = "checkbox";
    addCheckbox.className = "lexique-checkbox";
    addCheckbox.dataset.lexiconId = lexiconId;
    const checkboxTooltip = document.createElement("span");
    checkboxTooltip.className = "tooltip";
    checkboxTooltip.textContent = "Ajouter le mot à ce lexique";
    checkboxContainer.appendChild(addCheckbox);
    checkboxContainer.appendChild(checkboxTooltip);
    
    // Conteneur pour le bouton de surlignage (avec tooltip)
    const highlightButton = document.createElement("button");
    highlightButton.className = "tooltip-container lexique-highlight-toggle";
    if (active) {
      highlightButton.classList.add('active');
    }
    highlightButton.dataset.lexiconId = lexiconId;
    highlightButton.dataset.active = active ? "true" : "false";
    const feutreIcon = document.createElement("img");
    feutreIcon.src = "../assets/icons/feutre.png";
    feutreIcon.alt = "Feutre";
    feutreIcon.className = "feutre-icon";
    const highlightTooltip = document.createElement("span");
    highlightTooltip.className = "tooltip";
    highlightTooltip.textContent = "Activer/Désactiver le surlignage des mots du lexique";
    
    // Mise à jour du gestionnaire d'événements
    highlightButton.addEventListener("click", async () => {
      let currentState = highlightButton.dataset.active === "true";
      let newState = !currentState;
      try {
        await toggleLexiconHighlight(lexiconId, newState);
        highlightButton.dataset.active = newState ? "true" : "false";
        highlightButton.classList.toggle("active", newState);
        
        // Sauvegarder l'état dans le storage local
        const activeLexicons = Array.from(document.querySelectorAll('.lexique-highlight-toggle[data-active="true"]'))
          .map(btn => parseInt(btn.dataset.lexiconId));
        await browser.storage.local.set({ activeLexicons });
      } catch (error) {
        log("Erreur lors du toggle de surlignage pour le lexique", lexiconId, ":", error);
      }
    });
    
    highlightButton.appendChild(feutreIcon);
    highlightButton.appendChild(highlightTooltip);
    lexiqueDiv.appendChild(iconDiv);
    lexiqueDiv.appendChild(labelSpan);
    lexiqueDiv.appendChild(checkboxContainer);
    lexiqueDiv.appendChild(highlightButton);
    
    lexiquesContainer.appendChild(lexiqueDiv);
  }
  
  setTimeout(() => {
    const menu = document.getElementById("menu");
    if (!menu) return;
    const menuRect = menu.getBoundingClientRect();
    const containers = document.querySelectorAll('.tooltip-container');
    
    containers.forEach(container => {
      const tooltip = container.querySelector('.tooltip');
      if (!tooltip) return;
      tooltip.style.left = '50%';
      tooltip.style.transform = 'translateX(-50%) translateY(-5px)';
      
      const tooltipRect = tooltip.getBoundingClientRect();
      if (tooltipRect.left < menuRect.left) {
        const overflowLeft = menuRect.left - tooltipRect.left;
        tooltip.style.transform = `translateX(calc(-100% + ${overflowLeft}px)) translateY(-5px)`;
      } else if (tooltipRect.right > menuRect.right) {
        const overflowRight = tooltipRect.right - menuRect.right;
        tooltip.style.transform = `translateX(calc(-100% - ${overflowRight}px)) translateY(-5px)`;
      }
    });
  }, 100);
}

// Ajouter une fonction pour restaurer l'état des boutons au chargement
async function restoreHighlightingState() {
  try {
    const { activeLexicons } = await browser.storage.local.get("activeLexicons");
    if (activeLexicons && Array.isArray(activeLexicons)) {
      const buttons = document.querySelectorAll('.lexique-highlight-toggle');
      buttons.forEach(button => {
        const lexiconId = parseInt(button.dataset.lexiconId);
        const isActive = activeLexicons.includes(lexiconId);
        button.dataset.active = isActive ? "true" : "false";
        button.classList.toggle("active", isActive);
      });
    }
  } catch (error) {
    log("Erreur lors de la restauration de l'état des boutons:", error);
  }
}

function initModal() {
  log("initModal appelé");
  const modalOverlay = document.getElementById("modalOverlay");
  const modalFullText = document.getElementById("modalFullText");
  const closeModalBtn = document.getElementById("closeModalBtn");

  log("closeModalBtn =", closeModalBtn);

  if (!modalOverlay || !modalFullText || !closeModalBtn) {
    log("Les éléments modaux ne sont pas trouvés !");
    return;
  }

  closeModalBtn.addEventListener("click", () => {
    log("clic sur closeModalBtn !");
    closeDefinitionPopup();
  });
  modalOverlay.addEventListener("click", (event) => {
    if (event.target === modalOverlay) closeDefinitionPopup();
  });
}

document.addEventListener("DOMContentLoaded", initModal);

// ─────────────────────────────────────────────────────────────────────────────
// ▌ Bouton de connexion/déconnexion
// ─────────────────────────────────────────────────────────────────────────────
async function handleAuthButtonClick() {
  await browser.runtime.sendMessage({ action: "toggleAuth" });
  await refreshSidebarState();
}

// ─────────────────────────────────────────────────────────────────────────────
// ▌ Ajout d'un mot au(x) lexique(s)
// ─────────────────────────────────────────────────────────────────────────────
async function handleAddWordClick() {
  openBlock("menuContent");
  // 1) Vérifier la présence du token et du mot
  if (!authToken) {
    console.warn("⚠️ Pas de token d'authentification : impossible d'ajouter le mot.");
    return;
  }

  const selectedWordElement = document.getElementById("motSelectionne");
  const lexiconResultElement = document.getElementById("lexiconResult");

  if (!selectedWordElement) {
    console.warn("⚠️ Élément #motSelectionne introuvable.");
    return;
  }
  const selectedWord = selectedWordElement.textContent.trim();
  if (!selectedWord || selectedWord === "Aucun mot sélectionné") {
    console.warn("⚠️ Aucun mot à ajouter.");
    if (lexiconResultElement) lexiconResultElement.textContent = "Aucun mot à ajouter.";
    return;
  }

  // 2) Récupérer les IDs des lexiques sélectionnés
  const checkboxList = document.querySelectorAll("#lexiques .lexique-checkbox:checked");
  const selectedLexiconIds = Array.from(checkboxList).map(cb => parseInt(cb.dataset.lexiconId, 10));

  if (selectedLexiconIds.length === 0) {
    console.warn("⚠️ Aucun lexique sélectionné.");
    if (lexiconResultElement) lexiconResultElement.textContent = "Veuillez cocher au moins un lexique.";
    return;
  }

  // 3) Vérifier si le mot existe déjà dans l'un des lexiques sélectionnés
  let definitions = [];
  try {
    definitions = await fetchLexiconDefinitions(selectedWord);
  } catch (error) {
    log("Erreur lors de la récupération des définitions pour vérification :", error);
  }

  const existingLexiconIds = new Set();
  if (Array.isArray(definitions)) {
    for (const def of definitions) {
      if (selectedLexiconIds.includes(def.lexiconId)) {
        existingLexiconIds.add(def.lexiconId);
      }
    }
  }

  // 4) Si le mot existe déjà dans certains lexiques, on les affiche
  if (existingLexiconIds.size > 0) {
    if (lexiconResultElement) {
      lexiconResultElement.innerHTML =
        "Le mot <strong>" + selectedWord + "</strong> existe déjà dans le(s) lexique(s) : " +
        Array.from(existingLexiconIds).join(", ") + ".";
    }
  }

  // 5) Déterminer les lexiques où ajouter le mot
  const lexiconsToAdd = selectedLexiconIds.filter(id => !existingLexiconIds.has(id));
  if (lexiconsToAdd.length === 0) {
    return; 
  }

  // 6) Envoi d'une seule requête pour tous les lexiques restants
  try {
    log(`📡 Envoi de l'ajout du mot "${selectedWord}" dans les lexiques :`, lexiconsToAdd);
    const result = await window.AddWord(authToken, selectedWord, lexiconsToAdd, false);
    
    log("Réponse API :", result);

    // Rafraîchir l'UI et la liste des entrées
    await new Promise(resolve => setTimeout(resolve, 300));
    browser.runtime.sendMessage({ action: "refreshUI" });

    // 7) Affichage du message de succès
    if (lexiconResultElement) {
      lexiconResultElement.innerHTML +=
        "<br>✅ Mot <strong>" + selectedWord + "</strong> ajouté avec succès dans : " +
        lexiconsToAdd.join(", ") + ".";
    }

  } catch (error) {
    log("Erreur lors de l'ajout du mot :", error);
    if (lexiconResultElement) {
      lexiconResultElement.textContent = "Erreur lors de l'ajout : " + error.message;
    }
  }
}

// ─────────────────────────────────────────────────────────────────────────────
// ▌ Réception des messages
// ─────────────────────────────────────────────────────────────────────────────
browser.runtime.onMessage.addListener(async (message) => {
  log("📩 Message reçu dans sidebar.js :", message);
  
  if (message.action)
    switch (message.action) {
      case "refreshUI":
        log("🔄 Demande de rafraîchissement de la barre latérale.");
        await refreshSidebarState();
        break;

      case "mot_selectionne":
        if (message.selectedText) {
          log("🖋️ Mot sélectionné :", message.selectedText);
          const selectedWordElement = document.getElementById("motSelectionne");
          if (selectedWordElement) {
            selectedWordElement.textContent = message.selectedText;
          } else {
            console.warn("⚠️ Élément #motSelectionne introuvable.");
          }
          const lexiconResultElement = document.getElementById("lexiconResult");
          if (lexiconResultElement) {
            lexiconResultElement.innerHTML = "";
          }
          openBlock("etatContent");
        }
        break;

      case "getDefinition":
        if (message.selectedText) {
          log("📖 Recherche des définitions pour :", message.selectedText);
          openBlock("definitionContent");
          await window.showDefinitions(message.selectedText);
        }
        break;

      case "showDefinitions":
        if (Array.isArray(message.definitions)) {
          window.displayDefinitions(message.definitions);
        }
        break;

      case "fetchWiktionaryDefinitionResponse":
        if (message.selectedText) {
          log(`📖 Réception de la définition du Wiktionnaire pour '${message.selectedText}'`);
          window.displayDefinitions(message.definitions);
        }
        break;

      case "showLexiconResult":
        log("📚 Résultat des lexiques reçus :", message.lexicons);
        window.displayLexiconResults(message.lexicons);
        break;
      
      case "addWordResult":
        const lexiconResultElement = document.getElementById("lexiconResult");
        if (lexiconResultElement) {
          lexiconResultElement.innerHTML = message.lexicons;
        }
        break;

      case "addToLexicon":
        handleAddWordClick();
        break;
      
      case "openLexiconBlock":
        openBlock("menuContent");
        break;
        
      case "toggleAuth":
        break;

      case "authStatusChanged":
        break;

      case "updateUI":
        await refreshSidebarState();
        break;

      case "pyodide-simplemma-ready":
        return;

      case "saveToken":
        authToken = message.token;
        break;

      case "closeSidebarBlocks":
        closeBlock("menuContent");
        closeBlock("etatContent");
        closeBlock("definitionContent")
        break;
    }

  if (message.command) {
    switch (message.command) {
      case "activate-highlighting":
        const highlightButton = document.querySelector(`button[data-lexicon-id="${message.lexiconId}"]`);
        if (highlightButton) {
          highlightButton.dataset.active = "true";
          highlightButton.classList.add("active");
        }
        break;
        
      case "deactivate-highlighting":
        const highlightButtonOff = document.querySelector(`button[data-lexicon-id="${message.lexiconId}"]`);
        if (highlightButtonOff) {
          highlightButtonOff.dataset.active = "false";
          highlightButtonOff.classList.remove("active");
        }
        break;

      case "register-stats-script":
        break;

      case "register-highlighting-script":
        log("✅ Script de surlignage enregistré");
        browser.runtime.sendMessage({ 
          command: "updateAuthToken", 
          token: authToken 
        });
        break;

      case "updateAuthToken":
        authToken = message.token;
        window.authToken = message.token;
        break;

      default:
        console.warn("⚠️ Action inconnue reçue :", message.command);
    }
  }
});

// ─────────────────────────────────────────────────────────────────────────────
// ▌ DOMContentLoaded
// ─────────────────────────────────────────────────────────────────────────────
document.addEventListener("DOMContentLoaded", async () => {
  log("📦 DOM entièrement chargé. Initialisation de la sidebar.");
  authToken = await getAuthTokenFromStorage();
  window.authToken = authToken;
  await refreshSidebarState();

  const authButton = document.getElementById("auth-button");
  if (authButton) {
    authButton.addEventListener("click", handleAuthButtonClick);
  }

  const chercherDefButton = document.querySelector("#chercherDef");
  if (chercherDefButton) {
    chercherDefButton.addEventListener("click", async () => {
      openBlock("definitionContent");
      const selectedWord = document.getElementById("motSelectionne")?.textContent?.trim();
      if (selectedWord && selectedWord !== "Aucun mot sélectionné") {
        log(`📩 Recherche des définitions pour '${selectedWord}'`);
        await window.showDefinitions(selectedWord);
      } else {
        console.warn("⚠️ Aucun mot sélectionné pour la recherche.");
      }
    });
    log("✅ Bouton #chercherDef détecté dans le DOM.");
    chercherDefButton.style.pointerEvents = "auto";
    chercherDefButton.style.display = "block";
    chercherDefButton.style.visibility = "visible";
    chercherDefButton.disabled = false;
  } else {
    log("❌ ERREUR : Bouton #chercherDef introuvable.");
  }

  // Écouteur pour la case à cocher "toggle-definitions"
  const toggleCheckbox = document.getElementById("toggle-definitions");
  if (toggleCheckbox) {
    toggleCheckbox.addEventListener("change", (event) => {
      const showAll = event.target.checked;
      const lexiconContents = document.querySelectorAll("#mesLexiquesList .lexicon-content");
      lexiconContents.forEach(content => {
        if (showAll) {
          content.classList.remove("hidden");
        } else {
          content.classList.add("hidden");
        }
      });
    });
  }

  // Bouton "Ajouter le mot sélectionné"
  const addWordButton = document.getElementById("add-word-button");
  if (addWordButton) {
    addWordButton.addEventListener("click", handleAddWordClick);
  } else {
    console.warn("⚠️ Bouton #add-word-button introuvable dans le DOM.");
  }

  // Écouteur pour la case à cocher "toggle-definitions"
  const toggleButtons = document.querySelectorAll(".toggle-btn");
  toggleButtons.forEach((btn) => {
    if (!btn.textContent.trim()) {
      btn.textContent = "-";
      btn.style.fontSize = "15px";
    }
    btn.addEventListener("click", (event) => {
      event.stopPropagation();
      const header = btn.parentElement;
      const content = header.nextElementSibling;
      if (content) {
        content.classList.toggle("hidden");
        btn.textContent = content.classList.contains("hidden") ? "+" : "–";
      }
    });

    document.querySelectorAll('.block-content').forEach(block => {
      block.classList.add('hidden');
    });
  
    document.querySelectorAll('.toggle-btn').forEach(btn => {
      btn.textContent = '+';
      btn.style.fontSize = '15px';
  
      btn.addEventListener('click', (event) => {
        event.stopPropagation();
        const header = btn.parentElement;
        const content = header.nextElementSibling;
        if (content) {
          content.classList.toggle('hidden');
          btn.textContent = content.classList.contains('hidden') ? '+' : '–';
        }
      });
    });
  });
});

  document.querySelectorAll('.toggle-btn').forEach(btn => {
    btn.addEventListener('click', function() {
      const blockContent = this.parentElement.nextElementSibling;
      if (blockContent.classList.contains('hidden')) {
        blockContent.classList.remove('hidden');
        this.textContent = '–'; 
      } else {
        blockContent.classList.add('hidden');
        this.textContent = '+';
      }
    });
  });

async function toggleLexiconHighlight(lexiconId, isActive) {
  try {
    const button = document.querySelector(`button[data-lexicon-id="${lexiconId}"]`);
    if (button) {
      button.dataset.active = isActive.toString();
      button.classList.toggle('active', isActive);
    }

    await browser.tabs.query({ active: true, currentWindow: true }).then((tabs) => {
      browser.tabs.sendMessage(tabs[0].id, {
        command: isActive ? "activate-highlighting" : "deactivate-highlighting",
        lexiconId: lexiconId
      });
    });

    log(`✅ Surlignage ${isActive ? 'activé' : 'désactivé'} pour le lexique ${lexiconId}`);
  } catch (error) {
    log(`❌ Erreur lors du toggle du surlignage pour le lexique ${lexiconId}:`, error);
  }
}
