// Variables globales
window.activeLexiconIds = window.activeLexiconIds || new Set();

// Logs immédiats
log("🔵 DÉBUT DU FICHIER highlighting.js");
try {
    log("✅ highlighting.js chargé");
} catch (e) {
    log("❌ Erreur avec la fonction log:", e);
}

// Vérification de l'environnement
log("🔍 Vérification de l'environnement:", {
    hasBrowser: typeof browser !== 'undefined',
    windowLocation: window.location.href
});

// Gestion globale des erreurs
window.onerror = function(msg, url, line, col, error) {
    log("🔴 Erreur globale:", { message: msg, url: url, line: line, col: col, error: error });
    return false;
};

// Fonction d'initialisation du token depuis le stockage local
async function initAuthToken() {
    try {
        const { accessToken } = await browser.storage.local.get("accessToken");
        if (accessToken) {
            window.authToken = accessToken;
            authToken = accessToken;
            log("🔑 Token récupéré depuis le stockage local");
            return true;
        } else {
            log("⚠️ Aucun token trouvé dans le stockage local");
            return false;
        }
    } catch (error) {
        log("❌ Erreur lors de la récupération du token:", error);
        return false;
    }
}
initAuthToken();

// Écoute de mise à jour du token
browser.runtime.onMessage.addListener((message) => {
    if (message.command === "updateAuthToken" && message.token) {
        window.authToken = message.token;
        authToken = message.token;
        log("🔑 Token mis à jour via message:", !!message.token);
    }
});

(function () {
    try {
        if (window.hasRun) {
            log("⚠️ highlighting.js déjà chargé");
            return;
        }
        window.hasRun = true;
        
        // Variables internes
        let lexiconWordsCache = new Map();
        let highlightingActive = false;
        window.highlightingActive = false;
        let observer = null;
        
        // Gestion de la visibilité/page show pour réappliquer le surlignage
        document.addEventListener('visibilitychange', async () => {
            if (document.visibilityState === 'visible' && window.highlightingActive && activeLexiconIds.size > 0) {
                log("📄 Page redevenue visible, réinitialisation du surlignage");
                removeAllHighlights();
                await updateLexiconCache();
                highlightVisibleContent();
                attachMutationObserver();
            }
        });
        window.addEventListener('pageshow', async () => {
            if (window.highlightingActive && activeLexiconIds.size > 0) {
                log("📄 Page affichée (pageshow), réinitialisation du surlignage");
                removeAllHighlights();
                await updateLexiconCache();
                highlightVisibleContent();
                attachMutationObserver();
            }
        });
        
        // Gestion des messages reçus du background
        browser.runtime.onMessage.addListener((message, sender, sendResponse) => {
            log("📨 Message reçu:", message, "Context:", {
                highlightingActive,
                activeLexiconIds: Array.from(activeLexiconIds),
                hasAuthToken: !!window.authToken,
                hasGetAllLexiconWords: !!window.getAllLexiconWords
            });
            
            if (message.command === "activate-highlighting") {
                log(`🎯 Activation du surlignage pour le lexique ${message.lexiconId}`);
                startHighlighting(message.lexiconId)
                    .then(() => {
                        window.highlightingActive = true;
                        sendResponse(true);
                    })
                    .catch(error => {
                        log("❌ Erreur lors de l'activation:", error);
                        sendResponse(false);
                    });
                return true;
            }
            
            if (message.command === "deactivate-highlighting") {
                log(`🚫 Désactivation du surlignage pour le lexique ${message.lexiconId}`);
                stopHighlighting(message.lexiconId)
                    .then(() => {
                        if (activeLexiconIds.size === 0) {
                            window.highlightingActive = false;
                        }
                        sendResponse(true);
                    })
                    .catch(error => {
                        log("❌ Erreur lors de la désactivation:", error);
                        sendResponse(false);
                    });
                return true;
            }
            
            return false;
        });
        
        log("📡 Enregistrement du script auprès du background");
        browser.runtime.sendMessage({ command: "register-highlighting-script" });
        
        log("🚀 Initialisation de highlighting.js");
        
        // Fonction asynchrone pour mettre à jour le style d'un élément surligné en fonction des lexiques qui le concernent
        async function updateHighlightStyle(span, lexIds) {
            if (!lexIds || lexIds.length === 0) {
                span.style.backgroundColor = "rgba(255, 255, 0, 0.3)";
                span.style.backgroundImage = "";
                return;
            }
            if (lexIds.length === 1) {
                const hexColor = await getColorForLexicon(lexIds[0]);
                const rgbaColor = hexToRgba(hexColor, 0.3);
                span.style.backgroundColor = rgbaColor;
                span.style.backgroundImage = "";
            } else {
                const hexColors = await Promise.all(lexIds.map(id => getColorForLexicon(id)));
                const colors = hexColors.map(hex => hexToRgba(hex, 0.3));
                const n = colors.length;
                let stops = [];
                for (let i = 0; i < n; i++) {
                    const start = ((100 * i) / n).toFixed(2) + '%';
                    const end = ((100 * (i + 1)) / n).toFixed(2) + '%';
                    stops.push(`${colors[i]} ${start}, ${colors[i]} ${end}`);
                }
                const gradient = `linear-gradient(90deg, ${stops.join(', ')})`;
                span.style.backgroundImage = gradient;
                
            }
        }
        
        // Vérifier si un mot appartient à un lexique (à l'aide de la cache)
        function wordIsInLexicon(lexiconId, word) {
            const wordsSet = lexiconWordsCache.get(String(lexiconId));
            return wordsSet ? wordsSet.has(word.toLowerCase()) : false;
        }
        
        // Mise à jour incrémentale des éléments déjà surlignés lorsqu'un nouveau lexique est ajouté
        async function updateHighlightsForNewLexicon(newLexiconId) {
            const spans = document.querySelectorAll('.lexicon-highlight');
            for (let span of spans) {
                const word = span.textContent;
                if (wordIsInLexicon(newLexiconId, word)) {
                    let lexIds = [];
                    try {
                        lexIds = JSON.parse(span.getAttribute('data-lexicons'));
                    } catch (e) {
                        lexIds = [];
                    }
                    if (!lexIds.includes(newLexiconId)) {
                        lexIds.push(newLexiconId);
                        span.setAttribute('data-lexicons', JSON.stringify(lexIds));
                        await updateHighlightStyle(span, lexIds);
                    }
                }
            }
        }
        
        // Fonction startHighlighting modifiée pour intégrer la mise à jour incrémentale
        async function startHighlighting(lexiconId) {
            try {
                await initAuthToken();
                if (!window.authToken) {
                    throw new Error("❌ Pas de token d'authentification disponible");
                }
                if (lexiconId) {
                    if (!activeLexiconIds.has(lexiconId)) {
                        activeLexiconIds.add(lexiconId);
                        const activeLexicons = Array.from(activeLexiconIds);
                        await browser.storage.local.set({ activeLexicons });
                        log("📊 Lexiques actifs sauvegardés:", activeLexicons);
                        // Mise à jour de la cache pour inclure le nouveau lexique
                        await updateLexiconCache();
                        // Mise à jour immédiate des éléments surlignés pour intégrer le nouveau lexique
                        await updateHighlightsForNewLexicon(lexiconId);
                    }
                }
                window.highlightingActive = true;
                highlightingActive = true;
                highlightVisibleContent();
                attachMutationObserver();
                return true;
            } catch (error) {
                log("❌ Erreur lors du démarrage du surlignage:", error);
                window.highlightingActive = false;
                highlightingActive = false;
                throw error;
            }
        }
        
        async function stopHighlighting(lexiconId) {
            try {
                if (lexiconId) {
                    activeLexiconIds.delete(lexiconId);
                    const activeLexicons = Array.from(activeLexiconIds);
                    await browser.storage.local.set({ activeLexicons });
                    if (activeLexiconIds.size === 0) {
                        window.highlightingActive = false;
                        highlightingActive = false;
                        removeAllHighlights();
                        detachMutationObserver();
                    } else {
                        removeAllHighlights();
                        await updateLexiconCache();
                        highlightVisibleContent();
                    }
                } else {
                    window.highlightingActive = false;
                    highlightingActive = false;
                    activeLexiconIds.clear();
                    removeAllHighlights();
                    detachMutationObserver();
                }
                return true;
            } catch (error) {
                log("❌ Erreur lors de l'arrêt du surlignage:", error);
                throw error;
            }
        }
        
        // Mise à jour du cache des lexiques
        async function updateLexiconCache() {
            log("📥 updateLexiconCache - Début avec context:", {
                authToken: !!window.authToken,
                getAllLexiconWords: !!window.getAllLexiconWords,
                activeLexiconIds: Array.from(activeLexiconIds)
            });
            let allWords;
            try {
                if (!window.authToken) {
                    throw new Error("Pas de token d'authentification");
                }
                if (typeof window.getAllLexiconWords !== 'function') {
                    log("⚠️ getAllLexiconWords n'est pas une fonction");
                    log("Type de getAllLexiconWords:", typeof window.getAllLexiconWords);
                    log("Contenu de window.getAllLexiconWords:", window.getAllLexiconWords);
                    throw new Error("getAllLexiconWords n'est pas disponible");
                }
                log("📥 Appel de getAllLexiconWords...");
                allWords = await window.getAllLexiconWords(window.authToken);
                log("📝 Réponse de getAllLexiconWords:", allWords);
                if (!allWords || typeof allWords !== 'object') {
                    throw new Error(`Format de données invalide: ${JSON.stringify(allWords)}`);
                }
                lexiconWordsCache.clear();
                if (Object.keys(allWords).length === 0) {
                    log("⚠️ Aucun lexique reçu de getAllLexiconWords");
                    return false;
                }
                for (const [lexiconName, words] of Object.entries(allWords)) {
                    if (!Array.isArray(words)) {
                        console.warn(`⚠️ Format invalide pour le lexique ${lexiconName}:`, words);
                        continue;
                    }
                    const lexiconId = lexiconName.match(/\[(\d+)\]$/)?.[1];
                    if (!lexiconId) {
                        console.warn(`⚠️ Impossible d'extraire l'ID du lexique depuis: ${lexiconName}`);
                        continue;
                    }
                    log(`📎 Traitement du lexique ${lexiconName} (ID: ${lexiconId})`);
                    if (activeLexiconIds.has(Number(lexiconId))) {
                        lexiconWordsCache.set(lexiconId, new Set(words));
                        log(`📖 Lexique ${lexiconId} chargé avec ${words.length} mots`);
                    }
                }
                log("✅ Cache des lexiques mis à jour:",
                    Object.fromEntries([...lexiconWordsCache.entries()].map(([id, words]) => [id, [...words]])));
                return true;
            } catch (error) {
                log("❌ Erreur dans updateLexiconCache:", {
                    message: error.message,
                    stack: error.stack,
                    authTokenExists: !!window.authToken,
                    getAllLexiconWordsType: typeof window.getAllLexiconWords,
                    response: allWords
                });
                throw error;
            }
        }
        
        // Surlignage du contenu visible
        function highlightVisibleContent() {
            if (!highlightingActive) {
                log("⏸️ Surlignage inactif, sortie");
                return;
            }
            log("🔍 Début du surlignage du contenu visible");
            const BATCH_SIZE = 100;
            const BATCH_DELAY = 10; // ms
            const textNodes = [];
            const walker = document.createTreeWalker(
                document.body,
                NodeFilter.SHOW_TEXT,
                {
                    acceptNode: (node) => {
                        const parent = node.parentElement;
                        if (!parent || parent.closest('script, style, .lexicon-highlight') || !node.textContent.trim() || getComputedStyle(parent).display === 'none') {
                            return NodeFilter.FILTER_REJECT;
                        }
                        return NodeFilter.FILTER_ACCEPT;
                    }
                }
            );
            let node;
            while (node = walker.nextNode()) {
                textNodes.push(node);
            }
            log(`🔤 ${textNodes.length} nœuds de texte trouvés à traiter`);
            const processNextBatch = (startIndex) => {
                if (!highlightingActive || startIndex >= textNodes.length) {
                    return;
                }
                const endIndex = Math.min(startIndex + BATCH_SIZE, textNodes.length);
                const batch = textNodes.slice(startIndex, endIndex);
                batch.forEach(processTextNode);
                if (endIndex < textNodes.length) {
                    setTimeout(() => processNextBatch(endIndex), BATCH_DELAY);
                }
            };
            processNextBatch(0);
        }
        
        // Traitement d'un nœud de texte
        function processTextNode(textNode) {
            if (activeLexiconIds.size === 0) {
                log("⚠️ Aucun lexique actif, sortie du processTextNode");
                return;
            }
            const text = textNode.textContent;
            log(`🔍 Traitement du texte: "${text.substring(0, 50)}..."`);
            let lastIndex = 0;
            let fragments = [];
            const allWords = new Set();
            const matchedLexiconIdsMap = new Map();
            for (const [lexiconId, words] of lexiconWordsCache.entries()) {
                const numericId = parseInt(lexiconId);
                log(`🔄 Vérification du lexique ${lexiconId} (ID: ${numericId})`);
                if (activeLexiconIds.has(numericId)) {
                    log(`✅ Lexique ${lexiconId} actif, ajout de ${words.size} mots`);
                    words.forEach(word => allWords.add(word));
                    words.forEach(word => {
                        const lowerCaseWord = word.toLowerCase();
                        if (!matchedLexiconIdsMap.has(lowerCaseWord)) {  // Correction ici !
                            matchedLexiconIdsMap.set(lowerCaseWord, []);
                        }
                        matchedLexiconIdsMap.get(lowerCaseWord).push(lexiconId);
                    });
                }
            }
            log(`🔤 Nombre total de mots à rechercher: ${allWords.size}`);
            if (allWords.size === 0) {
                log("⚠️ Aucun mot à rechercher dans les lexiques actifs");
                return;
            }
            const wordsPattern = Array.from(allWords)
                .sort((a, b) => b.length - a.length)
                .map(escapeRegExp)
                .join("|");
            if (!wordsPattern) {
                log("⚠️ Aucun mot à rechercher, sortie");
                return;
            }
            const regex = new RegExp(`\\b(${wordsPattern})\\b`, "gi");
            let match;
            let matchCount = 0;
            while ((match = regex.exec(text)) !== null) {
                matchCount++;
                if (match.index > lastIndex) {
                    fragments.push(document.createTextNode(text.slice(lastIndex, match.index)));
                }
                const span = document.createElement("span");
                span.textContent = match[0];
                span.className = "lexicon-highlight";
                span.style.display = "inline-block";
                
                const matchedLexiconIds = matchedLexiconIdsMap.get(match[0].toLowerCase()) || [];
                span.setAttribute('data-lexicons', JSON.stringify(matchedLexiconIds));
                
                if (matchedLexiconIds.length === 0) {
                    span.style.backgroundColor = "rgba(255, 255, 0, 0.3)";
                } else {
                    updateHighlightStyle(span, matchedLexiconIds);
                }
                fragments.push(span);
                lastIndex = regex.lastIndex;
            }
            if (matchCount > 0) {
                log(`✨ ${matchCount} correspondances trouvées dans le nœud`);
            }
            if (lastIndex < text.length) {
                fragments.push(document.createTextNode(text.slice(lastIndex)));
            }
            if (fragments.length > 0) {
                const parent = textNode.parentNode;
                fragments.forEach(fragment => parent.insertBefore(fragment, textNode));
                parent.removeChild(textNode);
            }
        }
        
        // Suppression de tous les surlignages
        function removeAllHighlights() {
            log("🧹 Suppression de tous les surlignages");
            const highlights = document.querySelectorAll('.lexicon-highlight');
            log(`📊 ${highlights.length} surlignages à supprimer`);
            highlights.forEach(highlight => {
                const text = highlight.textContent;
                const textNode = document.createTextNode(text);
                highlight.parentNode.replaceChild(textNode, highlight);
            });
        }
        
        // Gestion des mutations DOM
        function attachMutationObserver() {
            log("👀 Attachement de l'observateur de mutations");
            let debounceTimer = null;
            const DEBOUNCE_DELAY = 250; // ms
            observer = new MutationObserver((mutations) => {
                if (debounceTimer) {
                    clearTimeout(debounceTimer);
                }
                debounceTimer = setTimeout(() => {
                    log(`🔄 Traitement groupé de ${mutations.length} mutations DOM`);
                    let shouldHighlight = false;
                    for (const mutation of mutations) {
                        if (mutation.type === 'childList') {
                            for (const node of mutation.addedNodes) {
                                if (node.nodeType === Node.ELEMENT_NODE && 
                                    !node.closest('.lexicon-highlight') && 
                                    node.textContent.trim()) {
                                    shouldHighlight = true;
                                    break;
                                }
                            }
                        }
                        if (shouldHighlight) break;
                    }
                    if (shouldHighlight) {
                        highlightVisibleContent();
                    }
                }, DEBOUNCE_DELAY);
            });
            observer.observe(document.body, {
                childList: true,
                subtree: true
            });
        }
        
        function detachMutationObserver() {
            if (observer) {
                log("👋 Détachement de l'observateur de mutations");
                observer.disconnect();
                observer = null;
            }
        }
        
        function escapeRegExp(string) {
            return string.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
        }
        
        function getLexiconIdFromName(lexiconName) {
            const match = lexiconName.match(/\[(\d+)\]$/);
            const id = match ? parseInt(match[1]) : null;
            log(`🏷️ Extraction de l'ID depuis '${lexiconName}': ${id}`);
            return id;
        }
        
        log("🔍 Vérification des dépendances au chargement:", {
            authToken: !!window.authToken,
            getAllLexiconWords: typeof window.getAllLexiconWords
        });
        
        async function checkAndRestoreHighlightingState() {
            try {
                const { activeLexicons } = await browser.storage.local.get("activeLexicons");
                if (!activeLexicons || !Array.isArray(activeLexicons) || activeLexicons.length === 0) {
                    window.highlightingActive = false;
                    highlightingActive = false;
                    return;
                }
                log("🔄 État des lexiques trouvé:", activeLexicons);
                for (const lexiconId of activeLexicons) {
                    await startHighlighting(lexiconId);
                }
            } catch (error) {
                log("❌ Erreur lors de la restauration de l'état:", error);
                window.highlightingActive = false;
                highlightingActive = false;
            }
        }
        
        checkAndRestoreHighlightingState();
        
    } catch (error) {
        log("🔴 Erreur critique dans l'IIFE:", error);
    }
})();
