importScripts("../utils/logger.js");
log("✅ pyodide_worker.js chargé avec succès !");

// URL de la version Pyodide la plus récente
const LATEST_BASE_URL = "https://cdn.jsdelivr.net/pyodide/v0.27.2/full/";

let pyodide = null;
let pyodideLoaded = false;      // Indique si Pyodide est en mémoire
let simplemmaLoaded = false;    // Indique si simplemma est déjà installé
let storedFrequencies = {};     // Stockage des fréquences accumulées

let autoAddEnabled = false;     // Ajout automatique désactivé par défaut
let isAuthenticated = false;    // Non connecté par défaut
let userThreshold = 10;         // Seuil par défaut
let trackedLanguages = [];      // Aucune langue suivie par défaut
let notifiedWords = {};         // Stockage des mots déjà signalés (pour éviter les doublons)
let stoplistFr = new Set();     // Stockage optimisé de la stoplist

// Écouteur des messages reçus du background script
self.onmessage = async (event) => {
  const data = event.data;
  log("[WebWorker] Message reçu du Background:", data);

  if (data.command === "pyodide-simplemma") {
    if (pyodideLoaded && simplemmaLoaded) {
      log("[Worker] Pyodide et Simplemma déjà chargés.");
      self.postMessage({ type: "pyodide-simplemma", status: "already_loaded", message: "Pyodide et Simplemma déjà en mémoire" });
      return;
    }
    try {
      if (!pyodideLoaded) {
        log("[Worker] Chargement de Pyodide...");
        try {
            importScripts(`${LATEST_BASE_URL}pyodide.js`);
            pyodide = await loadPyodide({ indexURL: LATEST_BASE_URL });
            await pyodide.loadPackage("lzma");
            await pyodide.loadPackage("micropip");
            pyodideLoaded = true;
        } catch (err) {
          log("[Worker] Erreur lors de l'import de pyodide.js :", err);
          self.postMessage({ type: "pyodide-simplemma", status: "error", message: err.toString() });
          return;
        }
        log("[Worker] Pyodide chargé avec succès !");
      }

      if (!simplemmaLoaded) {
        log("[Worker] Installation de simplemma...");

        // On encapsule la logique dans une fonction asynchrone pour faciliter l'usage d'await
        await pyodide.runPythonAsync(`
import micropip
import asyncio

async def main():
    print("Installation de simplemma...")
    await micropip.install("simplemma")
    print("Installation réussie.")
    import simplemma
    print("simplemma importé avec succès.")
    # Test simple : extraction de tokens et lemmatisation
    import re
    def tokenize(text):
        return re.findall(r"\\b\\w+\\b", text.lower())
    phrase = "Le chat mange la pomme"
    tokens = tokenize(phrase)
    print("Tokens extraits :", tokens)
    lemmatized_tokens = [simplemma.lemmatize(token, lang="fr") for token in tokens]
    print("Tokens lemmatisés :", lemmatized_tokens)
    return lemmatized_tokens

await main()
        `);
        simplemmaLoaded = true;
        log("[Worker] Simplemma installé avec succès !");
      }
      // Envoyer confirmation au background script
      self.postMessage({ type: "pyodide-simplemma", status: "success", message: "Pyodide et Simplemma chargés" });
    } catch (error) {
      log("[Worker] Erreur lors du chargement de Pyodide ou Simplemma :", error);
      self.postMessage({ type: "pyodide-simplemma", status: "error", message: error.toString() });
    }
  }

  // --- Traitement du texte envoyé par stats.js ---
  if (data.command === "process-text") {
    if (!pyodideLoaded || !simplemmaLoaded) {
      const errorMessage = "Les statistiques ne sont pas encore activées. Veuillez patienter..."
      log("[Worker] Pyodide non chargé.");
      self.postMessage({ type: "process-text", status: "error", message: errorMessage });
      return;
    }
    log("[Worker] Texte reçu pour analyse :", data.text);
    try {
      const stoplistArray = Array.from(stoplistFr);
      log("Stoplist utilisée :", stoplistArray);
      const result = await pyodide.runPythonAsync(`
import json
import re
import simplemma
from simplemma import langdetect

def detect_language(text):
    lang_scores = simplemma.langdetect(text, lang=("fr", "en", "es", "de", "it", "pt"))
    return lang_scores[0][0] if lang_scores else "unk"

def tokenize(text):
    return re.findall(r"\\b[a-zA-ZÀ-ÿ'-]+\\b", text.lower())

# Chargement de la stoplist
stoplist = set(json.loads('${JSON.stringify(stoplistArray)}'))

text = """${data.text.replace(/\"/g, '\\"')}"""
detected_lang = detect_language(text)
if detected_lang == "unk":
    detected_lang = "other"

tokens = tokenize(text)
if detected_lang == "fr":
    tokens = [token.lower().strip() for token in tokens if token.lower().strip() not in stoplist]

lemmatized_tokens = [simplemma.lemmatize(token, lang=detected_lang) for token in tokens]

freq = {}
for token in lemmatized_tokens:
    if token not in stoplist:
        freq[token] = freq.get(token, 0) + 1

json.dumps({"lang": detected_lang, "frequencies": freq}, ensure_ascii=False)
      `);
      const parsedResult = JSON.parse(result);
      const detectedLang = parsedResult.lang;
      if (!storedFrequencies[detectedLang]) {
        storedFrequencies[detectedLang] = {};
      }
      for (const [word, count] of Object.entries(parsedResult.frequencies)) {
        storedFrequencies[detectedLang][word] = (storedFrequencies[detectedLang][word] || 0) + count;
      }
      self.postMessage({ type: "update-frequencies", frequencies: storedFrequencies });
      if (autoAddEnabled) {
        checkThreshold(detectedLang);
      }
    } catch (error) {
      log("[Worker] Erreur dans l'analyse du texte :", error);
    }
  }

  if (data.command === "update-preferences") {
    userThreshold = data.threshold;
    trackedLanguages = data.trackedLanguages;
    autoAddEnabled = data.autoAdd;
    isAuthenticated = data.isAuthenticated;
    log("[Worker] Mise à jour des préférences :", { userThreshold, trackedLanguages, autoAddEnabled, isAuthenticated });
  }

  if (data.command === "update-stoplist") {
    stoplistFr = new Set(data.stoplist.map(word => word.toLowerCase().trim()));
    log("[Worker] Stoplist FR mise à jour :", stoplistFr);
  }
};

// --- Vérification du seuil et notification ---
function checkThreshold(lang) {
  if (!autoAddEnabled || !isAuthenticated) {
    log("[Worker] Auto-Add désactivé ou utilisateur non connecté. Aucune vérification de seuil.");
    return;
  }
  if (!trackedLanguages.includes(lang)) {
    log(`[Worker] La langue ${lang} n'est pas suivie.`);
    return;
  }
  log(`[Worker] Vérification des fréquences pour la langue ${lang}...`);
  let wordsAboveThreshold = {};
  if (storedFrequencies[lang]) {
    const exceededWords = Object.entries(storedFrequencies[lang])
      .filter(([word, count]) => count >= userThreshold && !(notifiedWords[lang] && notifiedWords[lang].includes(word)))
      .map(([word]) => word);
    if (exceededWords.length > 0) {
      if (!notifiedWords[lang]) {
        notifiedWords[lang] = [];
      }
      notifiedWords[lang].push(...exceededWords);
      wordsAboveThreshold[lang] = exceededWords;
    }
  }
  if (Object.keys(wordsAboveThreshold).length > 0) {
    log("[Worker] Nouveaux mots dépassant le seuil :", wordsAboveThreshold);
    self.postMessage({ type: "threshold-exceeded", wordsAboveThreshold: wordsAboveThreshold });
  } else {
    log("[Worker] Aucun nouveau mot n'a dépassé le seuil.");
  }
}
